﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[System.Serializable]
public class LogEntry
{
    public float timestamp;
    public Vector3 position;
    public Vector3 velocity;
    public string movementDirection;
    public float distanceSinceLastFrame;
    public float totalDistance;
    public float timeSinceLastCollectible;
    public int totalCollectibles;
    public int currentScore;
    public string rawUDPInput;

    public override string ToString()
    {
        string FormatVector(Vector3 v) => $"({v.x:F2}, {v.y:F2}, {v.z:F2})";

        return
            $"--- Log Entry ---\n" +
            $"Timestamp                  : {timestamp:F2} s\n" +
            $"Position                   : {FormatVector(position)}\n" +
            $"Velocity                   : {FormatVector(velocity)}\n" +
            $"Movement Direction         : {movementDirection}\n" +
            $"Distance Since Last Frame : {distanceSinceLastFrame:F2} units\n" +
            $"Total Distance             : {totalDistance:F2} units\n" +
            $"Time Since Last Collectible : {timeSinceLastCollectible:F2} s\n" +
            $"Total Collectibles         : {totalCollectibles}\n" +
            $"Current Score              : {currentScore}\n" +
            $"Raw UDP Input              : \"{rawUDPInput}\"\n" +
            $"----------------------------";
    }
}

public class LogManager : MonoBehaviour
{
    public static LogManager Instance;

    private List<LogEntry> logs = new List<LogEntry>();

    private Vector3 lastPosition;
    private float totalDistance = 0f;
    private float lastCollectibleTime = 0f;

    private int totalCollectibles = 0;
    private int currentScore = 0;

    private void Awake()
    {
        if (Instance != null && Instance != this)
        {
            Destroy(gameObject); 
            return;
        }

        Instance = this;
        DontDestroyOnLoad(gameObject);

        lastPosition = Vector3.zero;
    }

    public void Log(Vector3 currentPosition, Vector3 velocity, string direction, string rawUDP)
    {
        float timestamp = Time.time;
        float distance = Vector3.Distance(currentPosition, lastPosition);
        totalDistance += distance;

        var entry = new LogEntry
        {
            timestamp = timestamp,
            position = currentPosition,
            velocity = velocity,
            movementDirection = direction,
            distanceSinceLastFrame = distance,
            totalDistance = totalDistance,
            timeSinceLastCollectible = timestamp - lastCollectibleTime,
            totalCollectibles = totalCollectibles,
            currentScore = currentScore,
            rawUDPInput = rawUDP
        };

        logs.Add(entry);
        lastPosition = currentPosition;
        Debug.Log(entry.ToString());
    }

    public void RegisterCollectible(int scoreValue)
    {
        currentScore += scoreValue;
        lastCollectibleTime = Time.time;
    }

    public void RegisterTotalCollectible(int value)
    {
        totalCollectibles = value;
    }

    public void SaveLogsToFile(string filename)
    {
        string json = JsonUtility.ToJson(new LogWrapper { logs = logs }, true);
        System.IO.File.WriteAllText(Application.dataPath + "/" + filename, json);
    }

    [System.Serializable]
    private class LogWrapper
    {
        public List<LogEntry> logs;
    }
}
